package goldeneye.server.datamanager;

/**
 * haalt alle Buslijnen uit het leuvense op
 * 
 * DATABASE-layout: --> gebruikt bij return update() SERIAL id id gebruikt door
 * deLijn int number nummer van de lijn string(60) name naam van de lijn
 * 
 * @author CWA03
 */

// IMPORT-STATEMENTS
import goldeneye.server.DistanceCheck;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Date;
import java.text.SimpleDateFormat;

public class BusLineLoader extends DeLijnConnector {
	// VELDEN
	private ArrayList currentBusStops;

	private String currentBusStopId;

	private String currentBusStopName;

	private String currentBusStopUrl;

	private String currentLineHtml;

	private String currentLineName;

	private String currentLineNumber;

	private String currentLineLink;

	private String currentLineId;

	private String nextBusStopId;

	private String nextBusStopName;

	private String nextBusStopDuration;

	// CONSTRUCTOR
	public BusLineLoader(int refreshTime) {
		super(refreshTime);

		currentBusStopId = null;
		currentBusStopName = null;
		currentBusStopUrl = null;
		currentLineHtml = null;
		currentLineName = null;
		currentLineNumber = null;
		currentLineLink = null;
		currentLineId = null;
		nextBusStopName = null;
		nextBusStopDuration = null;
	}

	// METHODES

	/**
	 * Deze methode zorgt voor het herladen van de gegevens uit de database
	 * 
	 * @param dm
	 *            DataManager wordt gebruikt voor het wegschrijven van de
	 *            gegevens naar de database
	 * @return boolean true als de database succesvol werd geupdate
	 */
	public boolean update(DataManager dm) {
		// initeren van de lokale variabelen
		boolean succes = true;
		ArrayList linesData = null;

		// haal alle bushaltes uit de database op
		currentBusStops = dm.getData("cache", "delijn_haltes", "*");

		// verwijder de huidige data uit de database
		dm.clearData("cache", "delijn_buslijnen");

		// haal voor elke bushalte alle buslijnen op
		Iterator it1 = currentBusStops.iterator();

		while (it1.hasNext()) {
			String[] busStopData = (String[]) it1.next(); // deze array bevat
															// slechts 1 object

			currentBusStopId = busStopData[0];
			currentBusStopName = busStopData[1];

			createBusStopUrlPath(); // creeer de juiste busStopUrl

			linesData = getLinesOnCurrentBusStop();

			Iterator it2 = linesData.iterator();

			while (it2.hasNext()) {
				ArrayList lineData = (ArrayList) it2.next();
				
				// voeg de lijn slechts toe als de volgende halte in leuven ligt
				// ander criterium: afstand implementeren -> controleren of
				// halte voorkomt in bdatabase
				// oude markt
				// double om_lat = 50.87801906529983;
				// double om_lon = 4.6997880935668945;
				// deze afstand is verkeerd, is nog voor oude xml file te doen
				// werken
				// nieuwe afstand zou rond de 0.3 moeten liggen
				// double dist = 64.99;

				// gegevens omzetten naar doubles
				// TODO conversie van Lambert72, datum BD72 naar wsg84 (etrs89)
				// datum grs80
				//if (DistanceCheck.check(lineData.get(index_lon), 
				//		lineData.get(index_lat), om_lon, om_lat, dist)) {
				
				if (((String) lineData.get(5)).startsWith("LEUVEN")) {
					// System.out.println(lineData.get(1));
					// System.out.println(lineData.get(2));
					// System.out.println(lineData.get(3));
					// System.out.println(lineData.get(4));
					// System.out.println(lineData.get(5));
					if (!dm.setData("cache", "delijn_buslijnen", lineData)) {
						System.out
								.println("BusLineLoader - ERROR: fout bij het wegschrijven naar de database!");
						succes = false;
					} else {
						System.out.println("weggeschreven naar de database");
					}
				}
			}
		}

		return succes;
	}

	/**
	 * Deze methode haalt alle lijnen op op een bepaalde halte
	 * 
	 * @return ArrayList lijst met String die alle id van de lijnen op deze
	 *         halte bevat
	 */
	private ArrayList<ArrayList> getLinesOnCurrentBusStop() {
		// initieren van lokale variabelen
		ArrayList<ArrayList> lines = new ArrayList<ArrayList>();

		try {
			// haal de tabel met lijnen voor deze bushalte op
			StringBuffer pagina = getPage(currentBusStopUrl);
			String table = extractLineTable(pagina);

			// zoek de lijnen in deze tabel
			lines = extractLines(table, currentBusStopName);

			return lines;
		} catch (Exception e) {
			System.out.println(e.toString());
			return null;
		}
	}

	/**
	 * vormt een pad die verwijst naar de juiste pagina op de site van De Lijn
	 * 
	 */
	private void createBusStopUrlPath() {
		String datum = getDate();
		currentBusStopUrl = "/reisinfoServer/BediendHaltenaamServlet.do?datum="
				+ datum + "&haltenr=" + currentBusStopId;
	}

	/**
	 * geeft de huidige datum terug onder de vorm van yyyyMMdd
	 * 
	 * @return String stringrepresentatie vd tijd onder de vorm yyyyMMdd
	 */
	private String getDate() {
		Date date = new Date();
		SimpleDateFormat formatter;

		formatter = new SimpleDateFormat("yyyyMMdd");

		return (formatter.format(date));
	}

	/**
	 * Deze methode haalt de tabel met lijnen voor de opgegeven halte uit de
	 * pagina
	 * 
	 * @param pagina
	 *            StringBuffer representatie van de pagina die de tabel bevat
	 * @return String
	 */
	private String extractLineTable(StringBuffer pagina) {
		String tabel;
		int begin = 0, einde = 0;

		begin = pagina.indexOf("Dienstregeling</td></tr>") + 24;
		einde = pagina.indexOf("<tr><td colspan=\"4\" class=\"tdemright\">");
		tabel = pagina.substring(begin, einde);

		return tabel;
	}

	/**
	 * Deze methode extreheert de lijnen uit een tabel
	 * 
	 * @param table
	 * @return
	 */
	private ArrayList<ArrayList> extractLines(String table, String stop) {
		// initieren van de lokale variabelen
		ArrayList<ArrayList> lines = new ArrayList<ArrayList>();
		ArrayList<String> line = null;
		int begin = 0, einde = 0;

		do {
			begin = table.indexOf("<tr>", begin) + 4;
			einde = table.indexOf("</tr>", begin);
			currentLineHtml = table.substring(begin, einde);

			generateLineInformation();

			// stop de gegevens in een arrayList
			if (nextBusStopName != null) {

				line = new ArrayList<String>();

				line.add(currentBusStopId);
				line.add(currentBusStopName);
				line.add(currentLineNumber);
				line.add(currentLineName);
				line.add(nextBusStopId);
				line.add(nextBusStopName);
				line.add(nextBusStopDuration);

				lines.add(line);
			}
		} while (begin != 3);

		return lines;
	}

	/**
	 * Deze methode genereert de info voor elke lijn die opgeslagen worden in de
	 * velden van deze klasse.
	 */
	private void generateLineInformation() {
		getLineId();
		getLineNumber();
		getLineName();
		getLineLink();
		getNextBusStopData();
	}

	/**
	 * haal de id van de lijn uit de lijntabel
	 */
	private void getLineId() {
		int begin = currentLineHtml.indexOf("lijn=") + 5;
		int einde = begin + 4;

		currentLineId = currentLineHtml.substring(begin, einde);
	}

	/**
	 * haal het lijn nummer uit de lijntabel
	 */
	private void getLineNumber() {
		int begin = currentLineHtml.indexOf("<td>") + 4;
		int einde = currentLineHtml.indexOf("&nbsp;</td>");

		currentLineNumber = currentLineHtml.substring(begin, einde);
	}

	/**
	 * haal de naam van de lijn uit de lijntabel
	 */
	private void getLineName() {
		int begin = currentLineHtml.indexOf("&nbsp;</td><td>") + 15;
		int einde = currentLineHtml.indexOf("&nbsp;</td><td><a title=");

		currentLineName = currentLineHtml.substring(begin, einde);
	}

	/**
	 * haal de link met de tabel van de diensturen van deze tabel
	 */
	private void getLineLink() {
		String link;
		int beginAmp;
		int begin = currentLineHtml.indexOf("HREF=\"") + 6;
		int einde = currentLineHtml.indexOf("\">", begin);
		link = currentLineHtml.substring(begin, einde);

		while (link.indexOf("amp") != -1) {
			beginAmp = link.indexOf("amp");
			link = link.substring(0, beginAmp)
					+ link.substring(beginAmp + 4, link.length());
		}

		currentLineLink = link;
	}

	/**
	 * zoekt de volgende bushalte (drie haltes verder) als je de url van de
	 * desbetreffende lijn kent
	 */
	private void getNextBusStopData() {
		StringBuffer linePage = getPage(currentLineLink);
		String lineTrajectPage = getLineTrajectPage(linePage.toString());
		int begin, einde, indexCurrentBusStop;
		int timeCurrentBusStop;

		try {

			// zoek eerst de HUIDIGE BUSHALTE
			indexCurrentBusStop = lineTrajectPage.indexOf(currentBusStopName
					.toUpperCase());

			// als de bushalte om een of andere reden niet gevonden werd moet
			// dit niet uitgevoerd worden
			if (indexCurrentBusStop != -1) { // extra controle

				begin = indexCurrentBusStop - "</td><td>&nbsp;".length() - 5;
				timeCurrentBusStop = Integer.parseInt(lineTrajectPage
						.substring(begin, begin + 2))
						* 60
						+ Integer.parseInt(lineTrajectPage.substring(begin + 3,
								begin + 5));

				// zoek de VOLGENDE BUSHALTE (drie bushaltes verder) als ze
				// bestaat
				int index = 0;
				int indexToSearchFrom = indexCurrentBusStop;

				// ga naar de derde volgende bushalte!
				for (int i = 0; i < 3; i++) {
					index = lineTrajectPage.indexOf("<tr>", indexToSearchFrom);
					indexToSearchFrom = index + "<tr>".length();
					if (index == -1) {
						break;
					}
				}

				if (index != -1) { // extra controle
					// zoek de naam van de volgende bushalte
					begin = lineTrajectPage.indexOf("</td><td>&nbsp;", index) + 15;
					einde = lineTrajectPage.indexOf("</td></tr>", begin);

					if ((begin != -1) && (einde != -1)) { // extra controle
						nextBusStopName = lineTrajectPage.substring(begin,
								einde);

						getNextBusStopId();

						begin = begin - "</td><td>&nbsp;".length() - 5;

						// zoek de duur tot de volgende bushalte
						int timeNextBusStop = Integer.parseInt(lineTrajectPage
								.substring(begin, begin + 2))
								* 60
								+ Integer.parseInt(lineTrajectPage.substring(
										begin + 3, begin + 5));

						nextBusStopDuration = Integer.toString(timeNextBusStop
								- timeCurrentBusStop);
					} else {
						nextBusStopId = null;
						nextBusStopName = null;
						nextBusStopDuration = null;
					}
				} else {
					nextBusStopId = null;
					nextBusStopName = null;
					nextBusStopDuration = null;
				}
			} else {
				nextBusStopId = null;
				nextBusStopName = null;
				nextBusStopDuration = null;
			}

		} catch (StringIndexOutOfBoundsException e) {
			System.out.println("BusLineLoader - foutje! bushalte bestaat niet"
					+ e.toString());
		}
	}

	/**
	 * haalt de pagina met een lijntraject informatie op van de DeLijn site
	 * 
	 * @param linePage
	 *            String-representatie van de pagina met alle lijnen op 1
	 *            bushalte
	 * @return String String-representatie van pagina met lijntraject info
	 */
	private String getLineTrajectPage(String linePage) {
		// haal de url naar de pagina met het traject van de opgegeven lijn
		String link;
		int beginAmp;
		int begin = linePage.indexOf("<A TITLE=\"ritdetail\" HREF=\"") + 27;
		int end = linePage.indexOf("\">ritdetail</A></td>");

		link = linePage.substring(begin, end);

		while (link.indexOf("amp") != -1) {
			beginAmp = link.indexOf("amp");
			link = link.substring(0, beginAmp)
					+ link.substring(beginAmp + 4, link.length());
		}

		return getPage(link).toString();
	}

	/**
	 * Deze methode zoekt de id van de volgende bushalte, dit is opgeslagen in
	 * de database
	 */
	private void getNextBusStopId() {
		Iterator it = currentBusStops.iterator();
		boolean found = false;

		while (it.hasNext() && !found) {
			String[] busStopData = (String[]) it.next();
			String searchString = (busStopData[1].substring(7)).toUpperCase();
			if (nextBusStopName.contains(searchString)) {
				nextBusStopId = busStopData[0];
				found = true;
			} else {
				nextBusStopId = "";
			}
		}
	}
}
