package goldeneye.server.datamanager;

/**
 * Xml Parser om de bushaltes in te laden.
 * 
 * DATABASE layout: --> gebruikt bij return update() SERIAL id id gebruikt door
 * DeLijn string name naam van de halte double lat latitude van de halte double
 * lon longitude van de halte
 * 
 * @author CWA03
 */

// IMPORT-STATEMENTS
import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;

import goldeneye.server.DistanceCheck;
import goldeneye.server.game.Coordinate;

public class BusStopLoader extends DeLijnConnector {
	// VELDEN
	private static final String file = "/home/grim/busstopdata.xml"; // aanpassen!!

	// CONSTRUCTOR

	/**
	 * constructor
	 * 
	 * @param refreshTime
	 *            duur vooraleer de gegevens uit de database outdated zijn
	 */
	public BusStopLoader(int refreshTime) {
		super(refreshTime);
	}

	// METHODEN
	/**
	 * Deze methode zorgt ervoor dat de bushaltes in leuven in de database
	 * geupdate worden
	 * 
	 * @return boolean true als het updaten van de database geslaagd is
	 * @param db
	 *            DataManager-object gebruikt voor het wegschrijven van de data
	 */
	public boolean update(DataManager dm) {
		// initieren van de lokale variabelen
		boolean succes = true;

		// laadt het bestand in dat de informatie over de bushaltes bevat
		String xml = getXmlFile(file);
		// String xml = getXmlURL(file);

		// ophalen van de bushaltes en wegschrijven naar de database
		ArrayList<ArrayList> data = extractStops(xml);

		Iterator it = data.iterator();

		dm.clearData("cache", "delijn_haltes");

		while (it.hasNext()) {
			ArrayList busStopInfo = (ArrayList) it.next();
			String busStopName = (String) busStopInfo.get(1);
			// oud criterium
			if (busStopName.startsWith("Leuven")) {
			// nieuw criterium: afstand

			// oude markt
			// double om_lat = 50.87801906529983;
			// double om_lon = 4.6997880935668945;
			// deze afstand is verkeerd, is nog voor oude xml file te doen werken
			// nieuwe afstand zou rond de 0.3 moeten liggen
			// double dist = 64.99;
			

			// gegevens omzetten naar doubles
			// TODO conversie van Lambert72, datum BD72 naar wsg84 (etrs89) datum grs80
			double lon = Double.valueOf((String)busStopInfo.get(2)).doubleValue();
			double lat = Double.valueOf((String)busStopInfo.get(3)).doubleValue();
			// if (DistanceCheck.check(lon, lat, om_lon, om_lat, dist)) {
				// System.out.println("toegevoegd: "+ busStopName);
				if (!dm.setData("cache", "delijn_haltes", busStopInfo)) {
					System.out
							.println("BusStopLoader - ERROR: wegschrijven naar de database is mislukt!");
				}
			}

		}
		// data parsen naar voor het spel bruikbare tabel
		succes = parseBusStops(dm);

		return succes;
	}

	/**
	 * Deze methode laat een xml file als bestand
	 * 
	 * @param fileName
	 *            naam van het xml bestand
	 * @return String String dat het xml bestand bevat
	 */
	private String getXmlFile(String fileName) {
		try {
			FileReader fr = new FileReader(fileName);
			BufferedReader br = new BufferedReader(fr);

			String lijn;
			String file = "";
			while ((lijn = br.readLine()) != null) {
				file = file + lijn;
			}

			return file;
		} catch (IOException e) {
			System.out.println(e.toString());
			return null;
		}
	}

	/**
	 * Bushaltes extracten
	 * 
	 * @param file
	 *            String representatie van het xml-bestand
	 * @return ArrayList<ArrayList> lijst met alle informatie van alle
	 *         bushaltes
	 */
	private ArrayList<ArrayList> extractStops(String file) {
		// initieren van lokale variabelen
		ArrayList<ArrayList> busStopData = new ArrayList<ArrayList>();
		int start = 0;
		int end = 0;

		while (file.indexOf("<gml:featureMember>", start) != -1) {
			start = file.indexOf("<gml:featureMember>", start) + 19;
			end = file.indexOf("</gml:featureMember>", start);

			String stopData = file.substring(start, end);
			busStopData.add(createStop(stopData));

			start = end;
		}

		return busStopData;
	}

	/**
	 * bushalte creeren
	 * 
	 * @param data
	 */
	private ArrayList<String> createStop(String data) {
		String stopId = extractId(data);
		String stopName = extractName(data);
		Coordinate coord = extractCoords(data);

		ArrayList<String> busStopInfo = new ArrayList<String>();
		busStopInfo.add(stopId);
		busStopInfo.add(stopName);
		busStopInfo.add(Double.toString(coord.getLongitude()));
		busStopInfo.add(Double.toString(coord.getLatitude()));


		return busStopInfo;
	}

	/**
	 * haltenummer extracten
	 * 
	 * @param data
	 * @return
	 */
	private String extractId(String data) {
		int start = data.indexOf("<topp:STOP_ID>") + 14;
		int stop = data.indexOf("</topp:STOP_ID>", start);

		return data.substring(start, stop);
	}

	/**
	 * haltenaam extracten
	 * 
	 * @param data
	 * @return
	 */
	private String extractName(String data) {
		int start = data.indexOf("<topp:DESCRIPTIO>") + 17;
		int stop = data.indexOf("</topp:DESCRIPTIO>", start);

		return data.substring(start, stop);
	}

	/**
	 * haltecoordinaten extracten
	 * 
	 * @param data
	 * @return
	 */
	private Coordinate extractCoords(String data) {
		int start = data.indexOf("<gml:coordinates") + 82;
		int stop = data.indexOf("</gml:coordinates>", start);
		String coordString = data.substring(start, stop);
		String[] coordArray = coordString.split(",");

		double gisLat = Double.parseDouble(coordArray[0]);
		double gisLng = Double.parseDouble(coordArray[1]);

		return new Coordinate(gisLat, gisLng);
	}

	public boolean parseBusStops(DataManager dm) {
		boolean succes = true;
		
		// MAG TERUG UITGEVOERD WORDEN, nu naar nieuwe tabel delijn_haltes3
		// tabel kopiren en de dubbels eruit halen

		// kopie maken
		dm.delete("cache", "delijn_haltes3");
		dm.copy("cache", "delijn_haltes", "cache", "delijn_haltes3");

		// gegevens in arraylist steken
		ArrayList lijnen = new ArrayList();
		lijnen = dm.getData("cache", "delijn_haltes3", "id,\"naam_halte\"");
		
		// lijnen overlopen
		for (int i = 0; i < lijnen.size(); i++) {
			String[] current = new String[2];
			current = (String[]) lijnen.get(i);
			// matchen met de andere lijnen
			for (int j = i; j < lijnen.size(); j++) {
				// alle lijnen van elkaar verschillende index bekijken
				if (i != j) {
					if (((String[]) lijnen.get(j))[1].equals(current[1])) {
						// lijn verwijderen
						dm.delete("cache", "delijn_haltes3", "id",
								((String[]) lijnen.get(j))[0]);
					}
				}
			}
		}
		return succes;

	}
}
