package goldeneye.server.datamanager;

/**
 * Deze klasse verzorgt het up to date houden. De klasse maakt gebruik van BusLineLoader, BusStopLoader, 
 * DeLijnConnector, FlickrConnector, NewsreaderConnector 
 * 
 * @CWA03
 */

// IMPORT-STATEMENTS

import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Set;

import goldeneye.server.datamanager.Connector;

public class CronManager {
	//VELDEN
	
	// connectors
	HashMap<Connector,Calendar> connectors;			// sleutel: de connector, waarde: date wanneer het laatste upgedate 
	
	// databasemanager
	DataManager dm;
	
	// CONSTRUCTOR
	public CronManager(DataManager dm) {
		this.dm = dm;
	
		initializeConnectors();
		// updateConnectors();
	}

	/**
	 * zorgt voor het laden van alle connectors
	 */
	private void initializeConnectors() {
		connectors = new HashMap<Connector,Calendar>();
		Calendar calendar = Calendar.getInstance();
		
		// hashmap vullen met connectors
		FlickrConnector flickrConnector = new FlickrConnector(10*24);
		connectors.put(flickrConnector, calendar);
		
		NewsreaderConnector newsreaderConnector = new NewsreaderConnector(1);
		connectors.put(newsreaderConnector, calendar);
		
		BusStopLoader busStopLoader = new BusStopLoader(31*24);
		connectors.put(busStopLoader, calendar);
	}
	
	/**
	 * zorgt ervoor dat bij het opstarten de database gerefreshed wordt
	 */
	// private void updateConnectors() {
	//	Set connectorSet = connectors.keySet();
	//	Iterator it = connectorSet.iterator();
	//	while(it.hasNext()) {
	//		Connector connector = (Connector)it.next();
	//		connector.update(dm);
	//	}
	//}
	
	// METHODES
	/**
	 * zorgt ervoor dat de database geupdate wordt
	 */
	public void cronJobs() {
		// haal de connectors op
		Set connectorSet = connectors.keySet();
		Iterator it = connectorSet.iterator();
		while(it.hasNext()) {
			Connector connector = (Connector)it.next();
			if(!isUpToDate(connector)) {
				connector.update(dm);
			}
		}
	}
	
	/**
	 * kijkt na of de gegevens in de database up to date zijn
	 * 
	 * @return	boolean		true als de gegevens up to date zijn
	 */
	private boolean isUpToDate(Connector connector) {
		// lokale variabele initieren
		boolean upToDate = true;
		
		Calendar currentCalendar = Calendar.getInstance();
		long currentDateAsLong = currentCalendar.getTimeInMillis();					// tijd tot 1 januari 1970 in milliseconden
		Calendar calendarLastUpdated = connectors.get(connector);
		long dateLastUpdatedAsLong = calendarLastUpdated.getTimeInMillis();	        // tijd tot 1 januari 1970 in milliseconden
		
		long diff = currentDateAsLong - dateLastUpdatedAsLong;						// in milliseconden!!
		
		if(diff/(1000*60*60) >= (long)connector.getRefreshTime()) {						
			upToDate = false;
		}
		
		return upToDate;
	}
}

	