package goldeneye.server.datamanager;

/**
 * Deze klasse verzorgt voor alle connecties met de postgresql database
 * 
 * @author CWA03
 */

// IMPORT-STATEMENTS
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.postgresql.util.PSQLException;
import java.sql.Statement;
import java.util.ArrayList;

import goldeneye.server.datamanager.CronManager;

public class DataManager {

	// bijhouden met welke database er verbonden is tijdens het construeren
	private Connection db;

	// cronManager voor het updaten van de database
	private CronManager cronManager;

	/**
	 * Maakt een DataManger aan die de verbindingen met een POSTGRESQL database
	 * beheert.
	 * 
	 * @param url
	 *            De locatie van de POSTGRESQL-server. "Syntax:
	 *            jdbc:postgresql://hostname:poort/"
	 * @param username
	 *            Username waarmee ingelogd zal worden op de server
	 * @param password
	 *            Password waarmee ingelogd zal worden op de server
	 */
	public DataManager(String url, String username, String password) {
		// indien lege stings als gegevens, verbinden met de standaard db
		if (url.equals("") && username.equals("") && password.equals("")) {
			// indien nogdig: tunnel leggen naar postgresql server op
			// nijvel.cs.kuleuven.be
			url = "jdbc:postgresql://localhost:9090/";
			username = "server";
			password = "server";
		}

		// verbinden met de database
		try {

			// driver inladen
			Class.forName("org.postgresql.Driver");

			// proberen te verbinden
			db = DriverManager.getConnection(url, username, password);

		}
		// foutjes vangen
		catch (ExceptionInInitializerError e) {
			e.printStackTrace();
		} catch (LinkageError e) {
			System.out.println("Linkage error for org.postgresql.driver ");
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println("Class not found: org.postgresql.driver ");
			e.printStackTrace();
		} catch (SQLException e) {
			System.out.println("Database acces error");
			e.printStackTrace();
		}

		// maak een nieuwe cronManager
		//cronManager = new CronManager(this);
	}

	/**
	 * Getter of the property <tt>CronManager</tt>
	 * 
	 * @return Returns the cronManager.
	 */
	public CronManager getCronManager() {
		return cronManager;
	}

	/**
	 * Setter of the property <tt>CronManager</tt>
	 * 
	 * @param CronManager
	 *            The cronManager to set.
	 */
	public void setCronManager(CronManager cronManager) {
		this.cronManager = cronManager;
	}

	/**
	 * Data uit database ophalen
	 * 
	 * @param type
	 *            Het type van data dat je wil ophalen verwijzend naar het
	 *            PostGreSQL-schema in de database. In ons geval zal dit meestal
	 *            "cache" zijn.
	 * @param table
	 *            Naam van de tabel zaaruit je gegevens wenst op te halen, dit
	 *            is dezelfde naam als de naam van de tabel in de database.
	 * @param filter
	 *            De sql filter die in het SELECT statement moet komen. Hiermee
	 *            kan je bepaalde tabellen wegfilteren uit de output. Voorbeeld:
	 *            filter = "id,lat,lon"; Als de naam van een kolom een spatie of
	 *            een hoofdletter bevat moet de filter als volgt gebruiken:
	 *            filter = "id, \"naam halte\", lat, lon"
	 * @return De methode retourneert een ArrayList aangezien je niet weet
	 *         hoeveel lijnen je gaat terugkrijgen op voorhand. Elke item van
	 *         deze ArrayList is een array van Strings. De lengte van deze
	 *         arrays van Strings is gelijk aan het aantal kollo;en dat je
	 *         geretourneerd krijgt.
	 */
	public ArrayList getData(String type, String table, String filter) {
		ArrayList<String[]> out = null;
		// data ophalen uit de tabel
		try {
			out = new ArrayList<String[]>();
			// query uitvoeren
			Statement statement = db.createStatement();
			String sqlquery = "SELECT " + filter + " FROM \"" + type + "\".\""
					+ table + "\";";

			// uncomment volgende regel voor raw query output
			// System.out.println(sqlquery);
			ResultSet resultset = statement.executeQuery(sqlquery);

			// controleren of resultset niet leeg is
			if (resultset.getMetaData().toString() == "") {
				throw new Exception("ResultSetEmptyError");

			}
			// resultaat in arraylist steken
			// rijen overlopen
			while (resultset.next()) {
				String[] line = new String[resultset.getMetaData()
						.getColumnCount()];
				// kolommen overlopen
				for (int i = 1; i < resultset.getMetaData().getColumnCount() + 1; i++) {
					// uncomment volgende regel voor raw output
					// System.out.println(resultset.getString(i));
					line[i - 1] = (resultset.getString(i));
				}
				out.add(line);
			}
		} catch (SQLException e) {
			System.out.println("DataManager: fout bij verkrijgen info uit de database");
			out = null;
			// e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
			out = null;
		}

		return out;

	}

	/**
	 * Data in de database steken aan de hand van een ArrayList.
	 * 
	 * @param type
	 *            Het type van data dat je wil ophalen verwijzend naar het
	 *            PostGreSQL-schema in de database. In ons geval zal dit meestal
	 *            cache" zijn.
	 * @param table
	 *            Naam van de tabel zaaruit je gegevens wenst op te halen, dit
	 *            is dezelfde naam als de naam van de tabel in de database.
	 * @param data
	 *            Deze ArrayList bevat de Strings die in de database moeten
	 *            gestoken worden.
	 * 
	 * 
	 */
	public boolean setData(String type, String table, ArrayList data) {
		boolean out = true;
		try {
			Statement statement = db.createStatement();

			String sql = new String();
			sql = "'" + data.get(0) + "'";
			for (int i = 1; i < data.size(); i++) {
				sql += ", '" + data.get(i) + "'";
			}
			String sqlquery = "INSERT INTO \"" + type + "\".\"" + table
					+ "\" VALUES (" + sql + ");";
			// uncomment om de query controleren
			// System.out.println(sqlquery);
			statement.executeQuery(sqlquery);
		} catch (SQLException e) {

			if (checkError(e)) {
				e.printStackTrace();
				out = false;
			}
		}
		return out;

	}

	/**
	 * Data in de database steken aan de hand van een Array van Strings.
	 * 
	 * @param type
	 *            Het type van data dat je wil wijzigen verwijzend naar het
	 *            PostGreSQL-schema in de database. In ons geval zal dit meestal
	 *            "cache" zijn.
	 * @param table
	 *            Naam van de tabel waarin je gegevens wenst toe te voegen, dit
	 *            is dezelfde naam als de naam van de tabel in de database.
	 */
	public boolean setData(String type, String table, String[] data) {
		boolean out = true;
		try {
			Statement statement = db.createStatement();

			String sql = new String();
			sql = "'" + data[0] + "'";
			for (int i = 1; i < data.length; i++) {
				sql += ", '" + data[i] + "'";
			}
			String sqlquery = "INSERT INTO \"" + type + "\".\"" + table
					+ "\" VALUES (" + sql + ");";
			// uncomment om de query controleren
			// System.out.println(sqlquery);
			statement.executeQuery(sqlquery);
		} catch (SQLException e) {
			if (checkError(e)) {
				e.printStackTrace();
				out = false;
			}
		}
		return out;

	}

	/**
	 * Deze methode maakt de gespsecifieerde tabel leeg, zonder de structuur te
	 * verwijderen.
	 * 
	 * @param type
	 *            Het type van data dat je wilt leegmaken. Dit verwijst naar het
	 *            pgsql schema. In ons geval is dit meestal "cache".
	 * @param table
	 *            De naam van de tabel die je wilt leegmaken.
	 */
	public boolean clearData(String type, String table) {
		boolean out = true;

		try {
			Statement statement = db.createStatement();

			statement.executeQuery("DELETE FROM " + type + "." + table);
		} catch (SQLException e) {
			if (checkError(e)) {
				e.printStackTrace();
				out = false;
			}
		}
		return out;

	}

	/**
	 * Retourneert de beschikare modules in de database, met de volgende syntax:
	 * schema, table name
	 * 
	 */
	public ArrayList getModules() {
		ArrayList<String[]> modules = new ArrayList<String[]>();
		try {
			// query maken waarin de server wordt gevraagd naar alle tabellen
			Statement statement = db.createStatement();
			String sqlquery = "SELECT \"schemaname\",\"tablename\" FROM pg_tables WHERE tableowner = current_user;";
			// resultaat query vangen
			ResultSet resultset = statement.executeQuery(sqlquery);

			// en in een arraylist steken
			// rijen overlopen
			while (resultset.next()) {
				String[] line = new String[resultset.getMetaData()
						.getColumnCount()];
				// kolommen overlopen
				for (int i = 1; i < resultset.getMetaData().getColumnCount() + 1; i++) {
					// uncomment volgende regel voor raw output
					// System.out.println("test: "+resultset.getString(i));
					line[i - 1] = (resultset.getString(i));
				}
				modules.add(line);
			}

		} catch (Exception e) {
			e.printStackTrace();
			e.toString();
		}
		return modules;
	}

	/**
	 * Controleert of de exception doorgegeven moet worden of niet.
	 * 
	 * @param e
	 *            De sqlexception die gecontroleerd moet worden
	 * @return true als de exception doorgegeven moet worden, false als alles ok
	 *         is.
	 */
	private boolean checkError(Exception e) {
		boolean out = true;
		if ((e.toString()
				.equals("org.postgresql.util.PSQLException: No results were returned by the query."))
				|| (e.toString()
						.equals("org.postgresql.util.PSQLException: Geen resultaten werden teruggegeven door de query."))) {
			out = false;
		}

		return out;
	}

	/**
	 * Verwijdert een lijn uit een tabel
	 * 
	 * @param type
	 *            Het type van data dat de tabel die je wil verwijderen bevat
	 * @return true als het verwijderen succesvol was
	 * 
	 */
	public boolean delete(String type, String table, String colname, String key) {
		boolean succes = false;
		try {
			// query maken
			Statement statement = db.createStatement();
			String sqlquery = "DELETE FROM \"" + type + "\".\"" + table
					+ "\" WHERE " + colname + " = " + key;
			// uncomment voor query output
			// System.out.println(sqlquery);
			ResultSet resultset = statement.executeQuery(sqlquery);
		} catch (SQLException e) {
			if (checkError(e)) {
				e.printStackTrace();
				succes = false;
			}
		}
		return succes;

	}

	public boolean copy(String type1, String table1, String type2, String table2) {
		boolean succes = false;
		try {
			// query maken waarin
			Statement statement = db.createStatement();
			String sqlquery = "SELECT * INTO \"" + type2 + "\".\"" + table2
					+ "\" FROM \"" + type1 + "\".\"" + table1 + "\"";
			// uncomment voor query output
			System.out.println(sqlquery);
			ResultSet resultset = statement.executeQuery(sqlquery);
		} catch (SQLException e) {
			if (checkError(e)) {
				e.printStackTrace();
				succes = false;
			}
		}
		return succes;
	}

	/**
	 * Een tabel uit de database verwijderen
	 * 
	 * @param type
	 *            Het type van data dat de tabel die je wil verwijderen bevat
	 * @param table
	 *            De naam van de tabel
	 * @return true als het verwijderen succesvol was
	 */

	public boolean delete(String type, String table) {
		boolean succes = false;
		try {
			// query maken waarin
			Statement statement = db.createStatement();
			String sqlquery = "DROP TABLE \"" + type + "\".\"" + table + "\"";
			ResultSet resultset = statement.executeQuery(sqlquery);
			// uncomment voor query output
			// System.out.println(sqlquery);
		} catch (SQLException e) {
			if (checkError(e)) {
				e.printStackTrace();
				succes = false;
			}
		}
		return succes;

	}

	/**
	 * Data uit database ophalen
	 * 
	 * @param type
	 *            Het type van data dat je wil ophalen verwijzend naar het
	 *            PostGreSQL-schema in de database. In ons geval zal dit meestal
	 *            "cache" zijn.
	 * @param table
	 *            Naam van de tabel zaaruit je gegevens wenst op te halen, dit
	 *            is dezelfde naam als de naam van de tabel in de database.
	 * @param filter
	 *            De sql filter die in het SELECT statement moet komen. Hiermee
	 *            kan je bepaalde tabellen wegfilteren uit de output. Voorbeeld:
	 *            filter = "id,lat,lon"; Als de naam van een kolom een spatie of
	 *            een hoofdletter bevat moet de filter als volgt gebruiken:
	 *            filter = "id, \"naam halte\", lat, lon"
	 * @param filter2
	 *            De sql filter die achter de WHERE moet komen. Hiermee
	 *            sql-filters uitvoeren. Voorbeeld: filter2 = "nummer < 400 AND
	 *            name = 'matchname'";
	 * @return De methode retourneert een ArrayList aangezien je niet weet
	 *         hoeveel lijnen je gaat terugkrijgen op voorhand. Elke item van
	 *         deze ArrayList is een array van Strings. De lengte van deze
	 *         arrays van Strings is gelijk aan het aantal kollo;en dat je
	 *         geretourneerd krijgt.
	 */
	public ArrayList getData(String type, String table, String filter,
			String filter2) {
		ArrayList<String[]> out = null;
		// data ophalen uit de tabel
		try {
			out = new ArrayList<String[]>();
			// query uitvoeren
			Statement statement = db.createStatement();
			String sqlquery = "SELECT " + filter + " FROM \"" + type + "\".\""
					+ table + "\" WHERE " + filter2 + " ;";

			// uncomment volgende regel voor raw query output
			// System.out.println(sqlquery);
			ResultSet resultset = statement.executeQuery(sqlquery);

			// controleren of resultset niet leeg is
			if (resultset.getMetaData().toString() == "") {
				throw new Exception("ResultSetEmptyError");

			}
			// resultaat in arraylist steken
			// rijen overlopen
			while (resultset.next()) {
				String[] line = new String[resultset.getMetaData()
						.getColumnCount()];
				// kolommen overlopen
				for (int i = 1; i < resultset.getMetaData().getColumnCount() + 1; i++) {
					// uncomment volgende regel voor raw output
					// System.out.println(resultset.getString(i));
					line[i - 1] = (resultset.getString(i));
				}
				out.add(line);
			}
		} catch (SQLException e) {
			System.out.println("DataManager: fout bij verkrijgen info uit de database");
			out = null;
			// e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
			out = null;
		}

		return out;

	}

}
