package goldeneye.server.datamanager;

/**
 * Deze klasse haalt foto's van Flickr op. Je kan zoektermen gebruiken om een bepaalde foto op
 * te zoeken.
 * 
 * DATABASE layout: --> gebruikt bij return update()
 * 		DATETIME	last_up				wordt bijgehouden voor Time-based caching
 * 		double		lat					latitude van de foto
 * 		double		lng					longitude van de foto
 * 		text		streetname			straatnaam waar de foto genomen is
 * 		text		description			omschrijving van de foto
 * 		text		url					url waar de foto staat
 * 
 * @author	CWA03
 */

//IMPORT STATEMENTS
import com.aetrion.flickr.Flickr;
import com.aetrion.flickr.photos.PhotosInterface;
import com.aetrion.flickr.REST;
import com.aetrion.flickr.photos.PhotoList;
import com.aetrion.flickr.photos.SearchParameters;
import com.aetrion.flickr.photos.Photo;
import com.aetrion.flickr.photos.GeoData;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Date;

public class FlickrConnector extends Connector{

	// VELDEN
		
	private static String apiKey = "0a1d3c71f56244eb7fca1652f3e81c14";	 // nodig om te verbinden met Flickr
	private static String restHost = "www.flickr.com";
	private Flickr flickr;                                               // klasse die gebruikt wordt om met Flickr te communiceren
	private PhotosInterface photosInterface;
	private REST rest;													 // gebruikte methode om met Flickr te communiceren
	// private ArrayList<String> streetNames;								 // lijst met straatnamen om te gebruiken als zoekterm

	// CONSTRUCTOR
	/**
	 * @param	streetNames		straatnamen waarvan foto's moeten gezocht worden
	 */
	public FlickrConnector(int refreshTime) {
		super(refreshTime);
	
		try {
			// een verbinding met Flickr (api) opzetten
			rest = new REST();
			rest.setHost(restHost);
			flickr = new Flickr(apiKey, rest); 
			Flickr.debugStream = false;
			photosInterface = flickr.getPhotosInterface();
		} catch (Exception e) {  // excepties opvangen
			System.out.println("Error! --> initialisatie flickr");
		}
	}

	// METHODEN

	/**
	 * haalt nieuwe foto's op van Flickr
	 * 
	 * @return	boolean		true als de gegevens uit de database succesvol konden geupdate worden
	 */
	public boolean update(DataManager dm) {
		// verwijder de huidige informatie uit de database
		dm.clearData("cache", "flickr");
		// haal alle gridpoints voor de straatnamen op uit de database
		ArrayList gridpointsData = dm.getData("cache", "gridpoints", "*");
		
		ArrayList<String> photoInformation;				// hierin komt alle info over een bepaalde foto
		boolean succes = true;

		Iterator it1 = gridpointsData.iterator();

		// vul de ArrayList data met informatie voor iedere straat
		while(it1.hasNext()) {
			String[] gridpointData = (String[])it1.next();
			
			String streetNameGridpoint = gridpointData[3];
			String idGridpoint = gridpointData[0];
			
			PhotoList photoList = searchPhotos("leuven " + streetNameGridpoint);
			
			Iterator it2 = photoList.iterator();
			
			while(it2.hasNext()) {
				photoInformation = getInformationPhoto(idGridpoint, streetNameGridpoint,(Photo)it2.next());
				if((photoInformation.get(1)).contains(streetNameGridpoint)) {
					dm.setData("cache", "flickr", photoInformation);
				}
			}
		}
		
		return succes;
	}

	/**
	 * zoekt foto's op basis van de opgegeven zoektekst
	 * 
	 * @return	PhotoList  een lijst met mogelijke relevante foto's  
	 * @param	searchString	de zoekTekst bv. LEUVEN MUNTSTRAAT, null als een fout opgetreden
	 */
	private PhotoList searchPhotos(String searchString)
	{
		SearchParameters searchParameters = new SearchParameters();
		searchParameters.setText(searchString);
		//searchParameters.setSort(SearchParameters.INTERESTINGNESS_DESC);
		PhotoList photoList = null;
		
		try {
			photoList = photosInterface.search(searchParameters, 60, 0);
			return photoList;
		} catch (Exception e) {
			System.out.println("error" + e.toString());
			return null;
		}
	}

	/**
	 * deze methode haalt de gepaste informatie uit een Photo object
	 * 
	 * @param	idGridpoint			id gridpoint waar de foto bijhoort 
	 * @param	streetNameGridpoint	straatnaam gridpoint vanwaar de foto afkomstig is
	 * @param	photo				Photo-object dat alle informatie bevat
	 * @return	ArrayList<String>	allerlei info over de foto (zie bovenaan)
	 */
	private ArrayList<String> getInformationPhoto(String idGridpoint, String streetNameGridpoint, Photo photo) {
		ArrayList photoInformation = new ArrayList();
		
		// vul de ArrayList met informatie
			photoInformation.add(idGridpoint);
			photoInformation.add(streetNameGridpoint);
		
		// GeoData location = photo.getGeoData();
		// photoInformation.add(Double.toString(location.getLatitude()));
		// photoInformation.add(Double.toString(location.getLongitude()));
		
			photoInformation.add(photo.getTitle());
			photoInformation.add(photo.getLargeUrl());				// Dit kan nog aangepast worden naar getMediumUrl of getSmallUrl
		
		// geef de ArrayList terug
		return photoInformation;
	}
}
