package goldeneye.server.datamanager;

/**
 * kan het nieuws ophalen van verschillende nieuwsdiensten zoals google News, De Standaard, vrt-nieuws,...
 * 
 * DATABASE layout: --> gebruikt bij return update()
 * 		text		newsChannel		nieuwsdienst waarvan de rss feed komt
 * 		text		itemTitle		title van het nieuwsbericht
 * 		text 		itemDescription	korte omschrijving van het nieuwsbericht
 * 		date		itemDate		datum van het nieuwsbericht
 * 		text		itemLink		link naar het nieuwsbericht
 * 
 * @author CWA03
 */

// IMPORT STATEMENTS
import java.net.*;
import de.nava.informa.core.ChannelIF;
import de.nava.informa.core.ItemIF;
import de.nava.informa.impl.basic.ChannelBuilder;
import de.nava.informa.parsers.FeedParser;
import java.util.*;

public class NewsreaderConnector extends Connector {
	
	// VELDEN
	
	// bevat alle urls van de gebruikte nieuwsdiensten
	private ArrayList<URL> urls;
	// de kanalen voor elke nieuwsdienst
	private ArrayList<ChannelIF> channels;
	
	// CONSTRUCTOR
	public NewsreaderConnector(int refreshTime) {
		super(refreshTime);
		setRssFeeds();
		setChannels();
	}
	
	// METHODEN
	/**
	 * Deze methode zorgt dat alle nieuwsdiensten opgeslagen worden.
	 */
	private void setRssFeeds() {
		urls = new ArrayList<URL>();
		
		try {
			urls.add(new URL("http://rss.vrtnieuws.net/nieuwsnet_master/versie2/systeem/rss/nnII_nws_regionaal_vlaamsbrabant/index.xml"));
			// urls.add(new URL("http://news.google.be/nwshp?ie=UTF-8&oe=UTF-8&hl=nl&tab=wn&q=&output=rss"));	
		} catch(MalformedURLException e) {
			System.out.println("Newsreader: Fout gevormd URL-adres - " + e.toString());
		}
	}
	
	/**
	 * Deze methode laat voor elke nieuwsdienst een kanaal waarlangs rss-feeds kunnen opgehaald worden
	 */
	private void setChannels() {
		// initiren van de lokale variabelen
		channels = new ArrayList<ChannelIF>();
		ChannelIF kanaal;
		
		Iterator it = urls.iterator();
		while(it.hasNext()) {
			try {
				kanaal = FeedParser.parse(new ChannelBuilder(), (URL)it.next());
				channels.add(kanaal);
			} catch(Exception e){
				System.out.println("Newsreader: Kan kanaal niet laden - " + e.toString());
			}
			
		}
	}
	
	/**
	 * Deze methode zorgt voor het updaten van de informatie uit de database
	 * 
	 * @return boolean	true als het update geslaagd is
	 */
	public boolean update(DataManager dm) {
		// initieer de lokale variabelen
		Collection items;
		ChannelIF channel;
		ArrayList<String> itemInformation;
		ItemIF item;
		boolean succes = true;
		
		// verwijder de huidige data uit de database
		dm.clearData("cache", "news");
		
		// haal de items op
		Iterator it1 = channels.iterator();
		
		while(it1.hasNext()) {
			channel = (ChannelIF)it1.next();
			items = channel.getItems();
			
			Iterator it2 = items.iterator();
			
			while(it2.hasNext()){
				itemInformation = new ArrayList<String>();
				item = (ItemIF)it2.next();
				
				// informatie invullen
				itemInformation.add(channel.getTitle());
				itemInformation.add(item.getTitle());
				itemInformation.add(item.getDescription());
				itemInformation.add(item.getDate().toString());
				itemInformation.add(item.getLink().toString());
				
				String[] itemInformationString = new String[itemInformation.size()];
				for(int i = 0; i < itemInformation.size(); i++) {
					itemInformationString[i] = itemInformation.get(i);
				}
				
				// schrijf de gegevens weg naar de database
				if(!dm.setData("cache","news",itemInformation)) {
					System.out.println("NewsreaderConnector - ERROR: probleem bij het wegschrijven naar de database");
					succes = false;
				}
			}
		}
		
		return succes;
	}
}
