package goldeneye.server.game;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import java.awt.GridBagConstraints;
import java.lang.Math;
import java.util.ArrayList;
import goldeneye.server.datamanager.*;
import java.util.Iterator;
import goldeneye.server.Server;

/**
 * Deze component is een virtuele representatie van het bordspel, een soort van
 * sandbox waarin het spel zich afspeelt. Deze component houdt bij waar alle
 * pionnen staan, checkt de geldigheid van de huidige spelsituatie, en
 * co-ordineert de verschillende spelacties.
 */

public class Game {
	private static final int MAX_PLAYERS = 6;

	private static final int MAX_TURN_NUMBER = 24;
	//private MisterX misterX;

	private GameThread gameThread;

	// hashmaps
	private HashMap players;
	private HashMap items;
	private HashMap gridpoints;
	private HashMap leuvenaars;
	
	private ArrayList hints;
	private MisterX misterx;
	private ArrayList newsitems;
	private ArrayList newsdates;

	// overig
	private Time time; // gaan we dit ooit nog gebruiken ?
	private int turn;
	private int playersReady;
	private boolean finished; // 

	private String[] playerColors; // mogelijke pionkleuren voor playermarkers

	private boolean blnWaitingTurn;
	private String nowPlayingId;

	// datamanager
	DataManager db;

	// server-timer (om verbindingen te checken tijdens makeMove)
	private TimerThread mytimer;
	private boolean timerThreadContinue;		// variabele om de timer te laten stoppen/voetsdoen

	// server
	private Server myServer;
	
	public Game(Server srvr) {
		// de constructor set alle velden
		time = new Time();
		turn = 1;
		gridpoints = new HashMap();
		players = new HashMap();
		db = new DataManager("jdbc:postgresql://localhost:9090/", "server",	"server");
		playerColors = new String[] { "orange", "misterx", "purple", "blue", "gold", "green" };		
		hints = new ArrayList();
		newsitems = new ArrayList();
		newsdates = new ArrayList();
		
		myServer = srvr;
		
		initialiseGame();
	}

	// deze methode zorgt ervoor dat alle clients de juiste data
	// doorkrijgen: lijst met detectives, MisterX

	public void initialiseGame() {
		// gridpoints, players, items, leuvenaars genereren en in de velden
		// (HashMaps) steken
		// daarna moeten deze worden verzonden naar de verschillende verbonden
		// clients
		// daarna (als dit allemaal gelukt is) mag het commando runGame()
		// aangeroepen worden
		
		ArrayList gridList = db.getData("cache", "gridpoints", "*");
		Iterator it = gridList.iterator();
		int j = 0;
		
		while (it.hasNext()) {
			String[] row = (String[]) it.next();
			String name = row[3];
			double lng = Double.parseDouble(row[1]);
			double lat = Double.parseDouble(row[2]);
			GridPoint point = new GridPoint(name, "normal", new Coordinate(lat,
					lng));
			gridpoints.put("gridid" + j, point);
			j++;
		}

		ArrayList busList = db.getData("cache", "delijn_haltes2", "*");
		Iterator it2 = busList.iterator();
		while (it2.hasNext()) {
			String[] row = (String[]) it2.next();
			String id = row[0];
			String name = row[1];
			double lng = Double.parseDouble(row[3]);
			double lat = Double.parseDouble(row[2]);

			GridPoint point = new GridPoint(name, "bus", new Coordinate(lat,
					lng));
			gridpoints.put(id, point);
			//j++;
		}
		// bushaltes ophalen
		// bij gridpoint aanmaken : GridPoint point = new GridPoint(id, "bus",
		// new Coordinate(lat, lng));
		
		ArrayList newsList = db.getData("cache", "news", "*");
        Iterator it3 = newsList.iterator();
        while(it3.hasNext()){
                String[] row = (String[]) it3.next();
                String newsTitle = row[1];
                String newsDate = (String) row[3];
                newsitems.add(newsTitle);
                newsdates.add(newsDate);
        }

	}

	public void runGame() {
		
		System.out.println("Het spel is begonnen");
		
		finished = false;
		// het spelleke wordt gestart
		while (!finished) {
			
			System.out.println("\n");
			System.out.println("GAMESTATUS : Now playing round " + turn);
			startRound();
			dropInactivePlayers();
			//elke ronde een krantenbericht genereren
			sendNewsToClient();
			turn++; // voorlopig weten de clients zelf wel welke beurt het is.
			timerThreadContinue = false;
			
			// kijken of mister x niet ontsnapt is
			if (turn >= MAX_TURN_NUMBER) {
				endGame("misterxescaped");
			}
			
			// om de 3 rondes hints genereren
			if (turn%3 == 0 && turn > 0) {
				GridPoint curLoc = (GridPoint) gridpoints.get(misterx.getPosition());
				String curLocName = curLoc.getName();
				
				Hint hint = new Hint(turn, curLocName); // wieland verandert dit in de hint klasse ASAP
				hints.add(hint);
				addHintToStubs(hint);
			}
		}
	}
	
	public void startRound() {
		// deze methode itereert over de spelers en laat ze hun zet doen
		// het runnen van deze methode resulteert dus in dat iedereen 1 zet
		// gedaan heeft
		
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();
		for (int i = 0; i < keys.length; i++) {
			String id = (String) keys[i];
			
			if (players.containsKey(id)) {
				makeMove(id);

				blnWaitingTurn = true;
				while (blnWaitingTurn) {
					try {
						Thread.sleep(4000);
					} catch (InterruptedException e) {
						//e.printStackTrace();
					}
				}
			}
			
			if (finished == true) {
				break;
			}
		}
		
		System.out.println("GAMESTATUS: Round " + turn + " finished");
	}

	public void makeMove(String playerid) {
//		 1. COMMUNICATIE-DEEL

		// we overlopen de spelers.
		// speler die aan de beurt is: notificatie geven en startTurn()
		// uitvoeren op hem
		Player curplayer = (Player) players.get(playerid);
		curplayer.getClientStub().addChanges("itsyourturn", new String[] { "" });
		nowPlayingId = playerid;
		
		timerThreadContinue = true;
		mytimer = new TimerThread(this, curplayer);
		mytimer.start();

		// alle andere spelers
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();
		for (int i = 0; i < keys.length; i++) 
		{
			String id = (String) keys[i];
			Player player = (Player) players.get(id);
			if (!playerid.equals(id)) 
			{
				String[] data = new String[] { playerid }; 
				player.getClientStub().addChanges("nowplaying", data); // andere spelers notifyen wie er aan de beurt is
			}
		}

		// wachten op oproep vanuit endTurn() op de client

		// Timerthread starten.  Deze checkt of de speler de laatste 15 seconden nog gepolled heeft
		// en of zijn beurt nog niet meer dan 70 seconden bezig is, kwestie van als er iets misloopt met
		// 1 client, dat het hele spel niet hangt.
		// In beide gevallen wordt gewoon endTurn aangeroepen.  De speler zal bij het einde van de ronde 
		// verwijderd worden door dropInactivePlayers.
		
	}

	public void endMove(HashMap hsChanges) {
		
		System.out.println("TURN: player " + nowPlayingId + " finished his turn");
		timerThreadContinue = false; 		// om de timer te doen stoppen
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			//e.printStackTrace();
		}
		
		if (hsChanges != null && hsChanges.size() != 0) {
			Player me = (Player) players.get(nowPlayingId);
			String toId = null;
			String moveMethod = null;
			if (hsChanges.containsKey("move")) {
				Object[] moveData = (Object[]) hsChanges.get("move");
				toId = (String) moveData[0];
				moveMethod = "tevoet";
			} else if (hsChanges.containsKey("taximove")) {
				Object[] moveData = (Object[]) hsChanges.get("taximove");
				toId = (String) moveData[0];
				moveMethod = "taxi";
			} else if (hsChanges.containsKey("busmove")) {
				Object[] moveData = (Object[]) hsChanges.get("busmove");
				toId = (String) moveData[0];
				moveMethod = "bus";
			}
			// lokaal updaten
			me.setPosition(toId);
			// UPDATEN GEHEEL
			// de veranderingen die de speler doorvoerde ook bij de andere clients aanpassen		
			Set keySet = players.keySet();
			Object[] keys = keySet.toArray();
			for (int i = 0; i < keys.length; i++) {
				String id = (String) keys[i];
				Player player = (Player) players.get(id);
				ClientStub stub = player.getClientStub();
				if (! id.equals(nowPlayingId)) {
					stub.addChanges("moveplayers", new Object[] { nowPlayingId,	toId, moveMethod });
				}
			}
			
			// CHECKEN OF ER WINST IS		
			if (me.getPosition().equals(misterx.getPosition()) && ! me.getColor().equals("misterx")) {
				endGame("misterxcaught");
			}
			if (me.getColor().equals("misterx")) {
				for (int i=0; i < keys.length; i++) {
					String id = (String) keys[i];
					Player player = (Player) players.get(id);
					
					if (! id.equals(nowPlayingId) && player.getPosition().equals(me.getPosition())) {
						endGame("misterxcaught");
					}
				}
			}
		}
		
		blnWaitingTurn = false;
	}

	public void endGame(String message) {
		System.out.println("GAMESTATUS: Game ended : " + message);
		
		finished = true;			// geen nieuwe rondes meer beginnen
		blnWaitingTurn = false; 	// wachtende while-lus in startround laten verdergaan om gebreaked te worden
		
		myServer.destroyGame(message, nowPlayingId);
	}
	
	public String getMisterXID() {
		String misterXID = null;
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();

		for (int i = 0; i < keys.length; i++) {
			String id = (String) keys[i];
			Player player = (Player) players.get(id);

			if (misterx.getName().equals(player.getName())) {
				misterXID = id;
			}
		}
		return misterXID;
	}
	
	public void addPlayer(String newPlayerId, Player objNewPlayer) {
		players.put(newPlayerId, objNewPlayer);
	}

	// Grid-hashmap genereren om door te sturen naar de client
	public void removeMe (String id) {
		if (id.equals(nowPlayingId)) {
			endMove(null);			
		} 
		
		if (players.containsKey(id)) {
			Player leftPlayer = (Player) players.remove(id);
			System.out.println("Player " + leftPlayer.getName() + " left.");
		}
		
		Set keySet = (Set) players.keySet();
		Object[] keys = (Object[]) keySet.toArray();
		
		for (int i=0; i<keys.length; i++) {
			String id2 = (String) keys[i];
			Player player = (Player) players.get(id2);
			ClientStub stub = player.getClientStub();
			
			stub.addChanges("removeplayers", new Object[]{id});
		}
	}
	
	public HashMap generateGrid() {
		HashMap cl_gridMap = new HashMap();

		Object[] gridKeys = null;

		try {
			Set gridKeySet = (Set) gridpoints.keySet();
			gridKeys = gridKeySet.toArray();
		} catch (Exception e) {
			System.out.println(e);
		}

		for (int i = 0; i < gridKeys.length; i++) {
			String id = (String) gridKeys[i];
			GridPoint point = (GridPoint) gridpoints.get(id);

			String[] gridData = new String[4];
			gridData[0] = point.getName();
			Coordinate coord = point.getPosition();
			gridData[1] = point.getType();
			gridData[2] = "" + coord.getLatitude();
			gridData[3] = "" + coord.getLongitude();

			cl_gridMap.put(id, gridData);
		}

		return cl_gridMap;
	}

	public HashMap generatePlayers() {
		HashMap cl_playerMap = new HashMap();

		Set playerKeySet = (Set) players.keySet();
		Object[] playerKeys = (Object[]) playerKeySet.toArray();

		for (int i = 0; i < playerKeys.length; i++) {
			String id = (String) playerKeys[i];
			Player player = (Player) players.get(id);

			String[] playerData = new String[3];
			playerData[0] = player.getName();
			playerData[1] = player.getColor();
			playerData[2] = player.getPosition();

			cl_playerMap.put(id, playerData);
		}

		return cl_playerMap;
	}

	public ArrayList<String> generateTaxiIds(String pos) {
		double radius = 0.01;
		return getPossibleGridpoints(pos, radius);
	}

	public ArrayList<String> generateTevoetIds(String pos) {
		double radius = 0.005;
		return getPossibleGridpoints(pos, radius);
	}
	// TODO: generateBusIds implementeren
	// public HashMap generateBusIds (String pos) {;}
	public ArrayList generateBusIds (String pos) {
		ArrayList halteList = db.getData("cache", "delijn_buslijnen", "*", "id_huidige_bushalte = " + pos);
		
		if (halteList != null) {
			Iterator it = halteList.iterator();
		
			ArrayList busInfo = new ArrayList();
		
			while (it.hasNext()) {
				String[] row = (String[]) it.next();
				String lineName = row[3];
				String lineNumber = row[2];
				String nextId = row[4];
				String nextStopName = row[5];
				
				String lineInfo = nextStopName + ", lijn :  " + lineName + " (" + lineNumber + ")";
				
			
				String[] lineData = new String[]{nextId, lineInfo};
				busInfo.add(lineData);
			}
				
			return busInfo;
		} else {
			return new ArrayList();
		}
	}
	
	public ArrayList<String> getPossibleGridpoints(String pos, double radius) {
		GridPoint currentGridpoint = (GridPoint) gridpoints.get(pos);
		Set keySet = (Set) gridpoints.keySet();
		Object[] keys = (Object[]) keySet.toArray();

		ArrayList<String> validPoints = new ArrayList<String>();
		double currentLat = currentGridpoint.getPosition().getLatitude();
		double currentLon = currentGridpoint.getPosition().getLongitude();

		for (int i = 0; i < keys.length; i++) {
            String id = (String) keys[i];
			GridPoint grid = (GridPoint) gridpoints.get(id);

			double gridLat = grid.getPosition().getLatitude();
			double gridLon = grid.getPosition().getLongitude();

			double distance = Math
					.sqrt(((gridLat - currentLat) * (gridLat - currentLat))
							+ ((gridLon - currentLon) * (gridLon - currentLon)));

			if (distance < radius) {
				validPoints.add((String) keys[i]);
			}
		}
		return validPoints;
	}

	public HashMap setPlayerReady(String strPlayerId) {
		HashMap messages = new HashMap();

		Player objPlayer = (Player) players.get(strPlayerId);
		objPlayer.setReady();
		playersReady++;
		if (playersReady == 2) {			
			messages.put("gamestatus", new Object[]{"start"});

			// De andere stubs laten weten dat de nieuwe speler geadd moet worden
			addMeToOtherStubs(strPlayerId);
			
			// GameThread starten om de main thread vrij te houden, anders zou deze nooit meer polling-requests 
			// kunnen aannemen, aangezien hij vast zou zitten in startgame.
			gameThread = new GameThread(this);
			gameThread.start();
		} else if (playersReady < 2) {
			messages.put("gamestatus", new Object[]{"wait"});
		} else {
			messages.put("gamestatus", new Object[]{"join", nowPlayingId});
			addMeToOtherStubs(strPlayerId);
		}

		return messages;
	}

	public boolean readyFreddy(String strPlayerName) {
		try {
			if (! playerAlreadyExists(strPlayerName)) {
				if (players.size() == 1) {
					misterx = new MisterX(strPlayerName, getRandomGridId());
					addPlayer("misterx", misterx);
				} else {
					addPlayer("player" + players.size(), new Player(
							strPlayerName, getRandomGridId(),
							playerColors[players.size()]));
				}
				System.out.println("player " + strPlayerName + " joined");
				return true;
			} else {
				return false;
			}
		} catch (Exception e) {
			System.out.println("ReadyFreddy error : " + e);
			return false;
		}
	}

	private String getRandomGridId() {

		Set gridset = (Set) gridpoints.keySet();
		Object[] gridkeys = gridset.toArray();
		
		Set playerset = (Set) players.keySet();
		Object[] playerkeys = playerset.toArray();

		int randomNum;
		String randomId = null;
		boolean idIsValid = false;
		Random generator = new Random();
		
		while (! idIsValid) {
			randomNum = generator.nextInt(gridkeys.length);
			randomId = (String) gridkeys[randomNum];
			
			idIsValid = true;
			
			for (int i=0; i < playerkeys.length; i++) {
				Player player = (Player) players.get(playerkeys[i]);
				
				if (player.getPosition().equals(randomId)) {
					idIsValid = false;
				}
			}
		}
			
		return randomId;
	}

	public ClientStub getStub(String strClientId) {
		Player objPlayer = (Player) players.get(strClientId);
		return objPlayer.getClientStub();
	}

	public void addMeToOtherStubs(String myId) {
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();

		Player me = (Player) players.get(myId);
		String[] playerData = new String[4];
		playerData[0] = myId;
		playerData[1] = me.getName();
		playerData[2] = me.getColor();
		playerData[3] = me.getPosition();

		for (int i = 0; i < keys.length; i++) {
			String id = (String) keys[i];
			if (!id.equalsIgnoreCase(myId)) {
				Player player = (Player) players.get(id);
				ClientStub stub = player.getClientStub();
				stub.addChanges("addplayers", playerData);
			}
		}
	}
	
	public void dropInactivePlayers() {
		Date now = new Date();
		Set keyset = players.keySet();
		Object[] playerkeys = keyset.toArray();
		
		for (int i = 0; i < playerkeys.length; i++) {
			String currentkey = (String) playerkeys[i];
			Player curplayer = (Player) players.get(currentkey);
			ClientStub curstub = curplayer.getClientStub();
			
			Date then = (Date) curstub.getLastConnection();
			if (now.getTime() - then.getTime() > 15000) {
				Player player = (Player) players.remove(currentkey);
				if (player.getColor().equalsIgnoreCase("misterx")) {
					endGame("misterxleft");
				}
				playersReady--;
			}
		}
	}
	
	public String getNowPlayingId () {
		return nowPlayingId;
	}
	
	public Player getPlayer (String id) {
		return (Player) players.get(id);
	}
	
	private void addHintToStubs(Hint hint) {
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();
		
		for (int i=0; i<keys.length; i++) {
			String id = (String) keys[i];
			Player player = (Player) players.get(id);
			ClientStub stub = player.getClientStub();
			
			stub.addChanges("hints", new Object[]{hint.getText()});
		}
	}

	public void addChatToStubs (String speakerId, String message) {		
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();
		
		for (int i=0; i<keys.length; i++) {
			String id = (String) keys[i];
			
			if (! speakerId.equals(id)) {
				Player player = (Player) players.get(id);
				ClientStub stub = player.getClientStub();
			
				stub.addChanges("chat", new Object[]{speakerId, message});
			}
		} 
	}

	public boolean finished () {
		return finished;
	}
	
	public void substractPlayer () {
		playersReady--;
	}
	
	public int getNumberOfPlayers () {
		return playersReady;
	}
	
	public void setTimerThreadContinue (boolean ttCont) {
		timerThreadContinue = false;
	}
	
	public boolean getTimerThreadContinue () {
		return timerThreadContinue;
	}

	public boolean playerAlreadyExists (String newName) {
		Set keySet = (Set) players.keySet();
		Object[] keys = (Object[]) keySet.toArray();
		
		boolean nameAlreadyExists = false;
		
		for (int i=0; i<keys.length; i++) {
			String id = (String) keys[i];
			Player player = (Player) players.get(id);
			
			if (player.getName().equals(newName)) {
				nameAlreadyExists = true;
			}
		}
		
		return nameAlreadyExists;
	}
	
	public String getName (String id) {
		Player player = (Player) players.get(id);
		return player.getName();
	}
	
	private void sendNewsToClient()
	{
		Random generator = new Random();
		int r = generator.nextInt(newsitems.size());
		String newsTitle = (String) newsitems.get(r);
		String newsDate = (String) newsdates.get(r);
		addNewsTitleToStubs(newsTitle,newsDate);
	}


	private void addNewsTitleToStubs(String newsTitle, String newsDate){
		Set keySet = players.keySet();
		Object[] keys = keySet.toArray();

		for (int i=0; i<keys.length; i++) {
			String id = (String) keys[i];
			Player player = (Player) players.get(id);
			ClientStub stub = player.getClientStub();
		
			stub.addChanges("hints", new Object[]{"NIEUWSBERICHT: "+newsTitle+". (datum: "+newsDate+")"});
		}
	}
}